from iso20275 import Elf, get_csv_paths
import unittest
import csv


class TestElf(unittest.TestCase):

    def setUp(self):
        Elf.load(newest=True, cleaned=True)

    def test_dispatched_readers_work_same(self):
        "make sure single-dispatched implementation works"

        # items from default reading
        items1 = Elf.items()

        # construct & use a reader & get items from it
        path = get_csv_paths(newest=True, cleaned=True)[0]
        with path.open('r', newline='', encoding='utf-8') as csvfile:
            reader = csv.reader(csvfile, delimiter=',', quotechar='"', strict=True)
            next(reader)
            Elf.load(source=reader)
        items2 = Elf.items()

        # check results are same
        i1 = dict(items1)
        i2 = dict(items2)
        self.assertEqual(i1.keys(), i2.keys())
        for code in i1:
            names1 = [e.local_name for e in i1[code]]
            names2 = [e.local_name for e in i2[code]]
            self.assertEqual(names1, names2)

    def test_member_name_must_be_equal_to_elf_code(self):
        for elf, values in Elf.items():
            for value in values:
                self.assertEqual(elf, getattr(value, 'elf'))

    def test_number_of_elf(self):
        self.assertEqual(len(Elf), 3274)

    def test_254M_multiple_values(self):
        self.assertEqual(len(Elf['254M']), 2)

        self.assertEqual(Elf['254M'][0].local_name, '私人股份有限公司')
        self.assertEqual(Elf['254M'][0].alpha2, 'HK')
        self.assertEqual(Elf['254M'][0].transliterated_name, 'Si ren gu fen you xian gong si')

        self.assertEqual(Elf['254M'][1].local_name, 'Private company limited by shares')
        self.assertEqual(Elf['254M'][1].alpha2, 'HK')
        self.assertEqual(Elf['254M'][1].transliterated_name, 'Private company limited by shares')

    def test_F0A6_exists(self):
        self.assertTrue(len(Elf) > 0)

    def test_str(self):
        self.assertEqual(str(Elf['254M'][0]), '254M: 私人股份有限公司')
        self.assertEqual(str(Elf['254M'][0].local_name), '私人股份有限公司')

    def test_repr(self):
        self.assertEqual(repr(Elf['254M'][0]), "<iso20275.Elf.254M '私人股份有限公司'>")
        self.assertEqual(repr(Elf['J4JC'][0]), "<iso20275.Elf.J4JC 'Limited Partnership'>")

    def test_difference_cleaned_and_original(self):
        self.assertEqual(Elf['6W6X'][0].local_abbreviations, 'Co-operative Limited;Cooperative Limited;Co-op Limited;Coop Limited;Co-operative Ltd.;Cooperative Ltd.;Co-op Ltd.;Coop Ltd.;Co-operative;Cooperative;Co-op;Coop')
        Elf.load(newest=True, cleaned=False)
        self.assertEqual(Elf['6W6X'][0].local_abbreviations, 'Co-operative Limited;Cooperative Limited;Co-op Limited;Coop Limited;Co-operative Ltd.;Cooperative Ltd.;Co-opLtd.;Coop Ltd.;Co-operative;Cooperative;Co-op;Coop')

    def test_correct_csv_parsing(self):
        self.assertEqual(Elf['C7TI'][0].local_name, 'Ассоциации (союзы) садоводческих, огороднических и дачных некоммерческих объединений')
        self.assertEqual(Elf['C7TI'][0].transliterated_name, 'Assotsiatsii (soyuzy) sadovodcheskikh, ogorodnicheskikh i dachnykh nekommercheskikh ob"yedineniy')

    def test_to_load_different_versions(self):
        timestamps = ['2017-11-30', '2019-11-06', '2020-06-10', '2020-11-19', '2021-09-23']
        with_additional = [timestamps[1], timestamps[3]]
        for timestamp in timestamps:
             Elf.load(cleaned=False, additional=False, timestamp=timestamp)
             Elf.load(cleaned=True, additional=False, timestamp=timestamp)
             if timestamp in with_additional:
                Elf.load(cleaned=True, additional=True, timestamp=timestamp)


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

import ast
import sys


def readme():
    with open('README.rst', 'r', encoding='utf-8') as f:
        return f.read()


def get_version():
    filename = r'C:\Users\YHUBAUT\Documents\GitHub\iso-20275-python\iso20275\__init__.py'
    with open(filename, 'r') as f:
        tree = ast.parse(f.read(), filename)
        for node in tree.body:
            if (isinstance(node, ast.Assign) and
                    node.targets[0].id == '__version__'):
                version = ast.literal_eval(node.value)
        if isinstance(version, tuple):
            version = '.'.join([str(x) for x in version])
        return version


def get_install_requirements():
    return ['setuptools']


setup(
    name='iso-20275',
    version=get_version(),
    description='ISO 20275 Entity Legal Type package for Python',
    long_description=readme(),
    long_description_content_type='text/x-rst',
    license='MIT',
    author='Youri Hubaut',
    packages=find_packages(),
    package_data={'iso20275': [
        '*.csv'
    ]},
    url='https://github.com/Gawaboumga/iso-20275-python',
    keywords='internationalization i18n elf iso20275 entity legal types',
    install_requires=get_install_requirements(),
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: Stackless',
        'Topic :: Software Development :: Internationalization',
    ]
)



import csv
import re
from pathlib import Path
from collections import OrderedDict
from collections.abc import Iterable
from typing import Sequence, Union
from functools import singledispatch


__version__ = 0, 0, 17
__all__ = 'Elf',


class OrderedProperties(type):
    "metaclass for custom ordered properties access on Python 3.5 and earlier"

    @classmethod
    def __prepare__(mcs, name, bases):
        "provide order-keeping support"
        return OrderedDict()

    def __new__(cls, name, bases, classdict):
        "inject a custom __properties__ magic method"
        customclass = type.__new__(cls, name, bases, classdict)
        customclass.__properties__ = tuple(k for k,v in classdict.items() if isinstance(v, property))
        return customclass


class ElfEntry(metaclass=OrderedProperties):
    def __init__(self, data):
        self.__data = data

    @property
    def elf(self):
        return self._ElfEntry__data['elf']

    @property
    def country(self):
        return self._ElfEntry__data['country']

    @property
    def alpha2(self):
        return self._ElfEntry__data['alpha2']

    @property
    def jurisdiction(self):
        return self._ElfEntry__data['jurisdiction']

    @property
    def alpha2_2(self):
        return self._ElfEntry__data['alpha2_2']

    @property
    def local_name(self):
        return self._ElfEntry__data['local_name']

    @property
    def language(self):
        return self._ElfEntry__data['language']

    @property
    def language_code(self):
        return self._ElfEntry__data['language_code']

    @property
    def transliterated_name(self):
        return self._ElfEntry__data['transliterated_name']

    @property
    def local_abbreviations(self):
        return self._ElfEntry__data['local_abbreviations']

    @property
    def transliterated_abbreviations(self):
        return self._ElfEntry__data['transliterated_abbreviations']

    @property
    def creation_date(self):
        return self._ElfEntry__data['creation_date']

    @property
    def status(self):
        return self._ElfEntry__data['status']

    @property
    def modification(self):
        return self._ElfEntry__data['modification']

    @property
    def modification_date(self):
        return self._ElfEntry__data['modification_date']

    @property
    def reason(self):
        return self._ElfEntry__data['reason']


    def __str__(self):
        return self.elf + ': ' + self.local_name

    def __repr__(self):
        cls = type(self)
        return '<%s.Elf.%s %r>' % (
            cls.__module__, self.elf,
            self.local_name
        )


class ElfEntries:
    def __init__(self, line):
        self.__line = line

    def __getitem__(self, key):
        return self.__line[key]

    def __len__(self):
        return len(self.__line)

    def __iter__(self):
        return self


@singledispatch
def read_entries(source:Iterable):
    "produce list of ElfEntry items from a source, by default an iterator"
    table = {}
    columns = ElfEntry.__properties__

    for tokens in source:
        row = dict(zip(columns, tokens))
        entry = ElfEntry(row)
        if entry.elf in table:
            table[entry.elf].append(entry)
        else:
            table[entry.elf] = [entry]
    return table


@read_entries.register(Path)
def read_entries_from_csvpath(source:Path):
    "produce list of ElfEntry items from a filesystem Path pointing to a CSV file"
    with source.open('r', encoding='utf-8') as csvfile:
        next(csvfile)
        reader = csv.reader(csvfile, delimiter=',', quotechar='"', strict=True)
        entries = read_entries(reader)
    return entries


rgx = re.compile(r'(?i)^(?P<c>Cleaned)?([ -_]*(?P<a>with additional))?[ -_]*ISO-20275[ -_]*(?P<t>\d{4}-\d{2}-\d{2})\.csv$')

def get_csv_paths(newest=None, cleaned=None, additional=None, timestamp=None) -> Sequence[Path]:
    """get provided csv file paths, optionally just newest and/or
    cleaned only and/or ones with given timestamp"""

    csvpaths = Path(__file__).resolve().parent.glob('*.csv')
    tested = ((re.match(rgx, csvpath.name), csvpath) for csvpath in csvpaths)
    results = list((m.group('c'), m.group('a'), m.group('t'), csvpath) for m, csvpath in tested if m)

    if additional is True:
        if cleaned is True:
            results = (r for r in results if r[0] and r[1])
        elif cleaned is False:
            results = (r for r in results if not r[0] and r[1])
    else:
        if cleaned is True:
            results = (r for r in results if r[0] and not r[1])
        elif cleaned is False:
            results = (r for r in results if not r[0] and not r[1])
    results = list(results)

    if timestamp:
        results = (r for r in results if r[2] == timestamp)
    if newest:
        results = sorted(results, key=lambda r: r[2], reverse=True)
        results = (results[0],) if cleaned else results[:3]

    return tuple((r[-1] for r in results))


class MetaElf(type):

    def __new__(cls, name, bases, clsdct):
        elfclass = type.__new__(cls, name, bases, clsdct)

        # by default, the latest cleaned CSV source is used
        elfclass._codes = read_entries(get_csv_paths(newest=True, cleaned=True, additional=True)[0])
        return elfclass

    def __getitem__(cls, key):
        return ElfEntries(cls._codes[key])

    def __len__(cls):
        return len(cls._codes)

    def __iter__ (self):
        return self


class Elf(object, metaclass=MetaElf):

    @classmethod
    def load(cls, newest=None, cleaned=None, additional=None, timestamp=None, source:Union[Iterable,Path]=None):
        "load a particular CSV source (newest/cleaned/timestamp) or a custom file"
        if source is None:
            source = get_csv_paths(newest=newest, cleaned=cleaned, additional=additional, timestamp=timestamp)[0]
        cls._codes = read_entries(source)

    @classmethod
    def items(cls):
        return cls._codes.items()